<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Teknisa\MAN\Services;
use Teknisa\MAN\Utilities;

class exportacaoBalancaToledo extends CrudWithParameterBag {

    protected $ExportacaoBalancaToledo;
    protected $utilService;

    public function __construct(Manager $dataSourceManager, Services\ExportacaoBalancaToledo $ExportacaoBalancaToledo, Utilities\Util $utilService, ParameterBag $parameterBag) {
        $this->ExportacaoBalancaToledo = $ExportacaoBalancaToledo;
        $this->utilService = $utilService;
        parent::__construct($dataSourceManager, $parameterBag);
    }

    public function findReport(DTO\Request\Filter $request, DTO\Response $response) {

        $conditions = $this->utilService->getParamsAsArray($request);
        $this->buildParams($conditions);
        $result = [];
        if (isset($conditions['IDTABEPRECGER'])) {
            $result = $this->ExportacaoBalancaToledo->find($conditions['CDFILIAL'], $conditions['CDTABEPREC'], $conditions['LISTA'], $conditions['TODOS'], $conditions['CDARVPRODI'], $conditions['CDARVPRODF'], $conditions['IDPESAPROD'], $conditions['IDTABEPRECGER']);
        } else {
            $result = $this->ExportacaoBalancaToledo->find($conditions['CDFILIAL'], $conditions['CDTABEPREC'], $conditions['LISTA'], $conditions['TODOS'], $conditions['CDARVPRODI'], $conditions['CDARVPRODF'], $conditions['IDPESAPROD']);
        }
        $response->addDataSet(new DataSet("vnd44000_exportacaoBalancaToledo", $result));
        return $response;
    }

    public function buildParams(&$conditions){
        if(!isset($conditions['INTERVALISTA'])){
            $conditions['INTERVALISTA'] = null;
        }
        if ($conditions['INTERVALISTA'] == 'I') {
            $conditions['LISTA'] = null;
            $conditions['TODOS'] = null;

        }
        else if ($conditions['INTERVALISTA'] == 'L'){
            $conditions['LISTA'] = implode('_', $conditions['LISTA']);
            $conditions['TODOS'] = null;
            $conditions['CDARVPRODI'] = null;
            $conditions['CDARVPRODF'] = null;
        }
        else {
            $conditions['LISTA'] = null;
            $conditions['TODOS'] = $conditions['INTERVALISTA'];
            $conditions['CDARVPRODI'] = null;
            $conditions['CDARVPRODF'] = null;
        }
    }

    public function exportarArq(DTO\Request\Filter $request, Response $response) {
        $params     = $request->getParameters()["Filter"];
        $conditions = [];
        foreach ($params as $filter) {
            $conditions[$filter['name']] = $filter['value'];
        }
        $this->buildParams($conditions);
        $result = [];
        if(isset($conditions['IDTABEPRECGER'])) {
            $result = $this->ExportacaoBalancaToledo->find($conditions['CDFILIAL'], $conditions['CDTABEPREC'], $conditions['LISTA'], $conditions['TODOS'], $conditions['CDARVPRODI'], $conditions['CDARVPRODF'], $conditions['IDPESAPROD'], $conditions['IDTABEPRECGER']);
        } else {
            $result = $this->ExportacaoBalancaToledo->find($conditions['CDFILIAL'], $conditions['CDTABEPREC'], $conditions['LISTA'], $conditions['TODOS'], $conditions['CDARVPRODI'], $conditions['CDARVPRODF'], $conditions['IDPESAPROD']);
        }

        $text   = '';
        foreach ($result as $value) {
            $text .= $value['TEXTO'] . "\n";
        }

        $resultTXTINFO = $this->ExportacaoBalancaToledo->exportTXTINFO();
        $textTXTINFO   = '';
        foreach ($resultTXTINFO as $valueTXTINFO) {
            $textTXTINFO .= $valueTXTINFO['TEXTO'] . "\n";
        }

        $nomeArq      = 'ItensMgv';
        $nomeTXTINFO  = 'TXINFO';
        $nomeInfNutri = 'InfNutri';
        $contentInfNutri = $this->exportArquivoNutricional();

        $return['texto']         = $text;
        $return['textotxinfo']   = $textTXTINFO;
        $return['InfNutri']      = $contentInfNutri;
        $return['nome']          = $nomeArq;
        $return['nomeTXTINFO']   = $nomeTXTINFO;
        $return['nomeInfNutri']  = $nomeInfNutri;
        $return['error']         = false;


        $response->addDataSet(new DataSet('resultado', $return));
    }

    public function findInfoExtra(DTO\Request\Filter $request, DTO\Response $response) {

        $this->dataSourceName = "info_extra";
        parent::find($request, $response);
    }
    
    public function saveInfoExtra(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = "info_extra";
        $dataSet = $request->getDataSet();
        $rows = $dataSet->getRows();
        try {
    	    if($rows[0]['__is_new']) {
                $this->ExportacaoBalancaToledo->saveInfoExtra($rows);
    	    } else {
    	        $this->ExportacaoBalancaToledo->updateInfoExtra($rows);
    	    }
        } catch (Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteInfoExtra(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $params = $this->utilService->getParamsAsArray($request);
            $this->ExportacaoBalancaToledo->deleteInfoExtra($params);
        } catch (\Exception $e) {
            $message = 'Não é possível remover uma informação que está atualmente atribuída à um produto!';
            $response->setError(new DTO\Response\Error($message, $e->getCode()));
        }
    }

    /**
     * Salva Cadastramento Automattico
     * @param Row $row
     * @param Response $response
     */
    public function salvaCadastramentoAutomatico(DTO\Request\Row $row, DTO\Response $response)
    {
        try {

            $row  = $row->getRow();
            $this->ExportacaoBalancaToledo->salvaCadastramentoAutomatico($row);

        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function findTabelaPrecoGeral(DTO\Request\Filter $request, DTO\Response $response) {
        $result = [];
        $conditions = $this->utilService->getParamsAsArray($request);
        
        if(isset($conditions['CDFILIAL'])) {
            $result = $this->ExportacaoBalancaToledo->findTabelaPrecoGeral($conditions);
            
            $response->addDataSet(new DataSet("getTabelaDePrecoGeralUnidade", $result));
        }
        
        return $result;
    }
    
    public function exportArquivoNutricional(){
        $query = $this->ExportacaoBalancaToledo->exportArquivoNutricional();
        // fixos
        $indicador = 'N';
        $reservado = 'A';
        
        // file
        $content = '';
        $fileName = 'Infnutri.txt';
        
        foreach($query as $row){
            
            // parametro C, representa o código de informação nutricional e deve ter 6 caracteres
            $codigoInfo = $this->checkField($row, 'CDINFORMNUTRI', 6);
            // parametro B, representa a quantidade e deve ter 3 caracteres
            $quantidade = $this->checkField($row, 'QTPORCAOVND', 3);
            
            // parametro D, representa a unidade da porção e deve ter 1 caracter
            $unidade = $this->checkField($row, 'IDUNPORCAOVND', 1);
            
            // parametro E, representa a parte inteira da medida caseira e deve ter 2 caracteres
            $parteInteira = $this->checkField($row, 'QTMEDCASVND', 2);
            
            // parametro F, representa a parte decimal da medida caseira e deve ter 1 caracter
            $parteDecimal = $this->checkField($row, 'IDPADEMEDCASVND', 1);
            
            // parametro G, representa a medida caseira utilizada e deve ter 2 caracteres
            $medidaCaseira = $this->checkField($row, 'IDTIPOMEDCASVND', 2);
            
            // parametro E, representa o valor energetico e deve ter 4 caracteres
            $energetico = $this->checkField($row, 'VRENRGNUTRI', 4);
            
            // parametro I, representa os carboidatros e deve ter 4 caracteres
            $carboidatros = $this->checkField($row, 'QTDCARBNUTRI', 4);
            
            // parametro L, representa as proteinas e deve ter 3 caracteres
            $proteinas = $this->checkField($row, 'QTDPROTNUTRI', 3);
            
            // parametro N, representa gorduras totais e deve ter 3 caracteres
            $gordurasTotais = $this->checkField($row, 'QTDGTOTNUTRI', 3);
            
            // parametro O, representa gorduras saturadas e deve ter 3 caracteres
            $gordurasSaturadas = $this->checkField($row, 'QTDGSATNUTRI', 3);
            
            // parametro P, representa gorduras trans e deve ter 3 caracteres
            $gordurasTrans = $this->checkField($row, 'QTDGTRANUTRI', 3);
            
            // parametro Q, representa fibra alimentar e deve ter 3 caracteres
            $fibras = $this->checkField($row, 'QTDFIBRANUTRI', 3);
            
            // parametro U, representa sodio e deve ter 5 caracteres
            $sodio = $this->checkField($row, 'QTDSODNUTRI', 5);
            
            $line = $indicador .$codigoInfo .$reservado .$quantidade .$unidade .$parteInteira .$parteDecimal .$medidaCaseira .$energetico .$carboidatros .$proteinas .$gordurasTotais .$gordurasSaturadas .$gordurasTrans .$fibras .$sodio .PHP_EOL;
            
            $content .= $line;
        }
        
        return $content;
        
        /* 
        Todos os campos:
            N		(1)	- Indicador nova Informação nutricional, sempre 'N' 
            CCCCCC	(6)	- Código da Informação Nutricional (Campo: CDINFORMNUTRI)
            A		(1)	- Reservado
            BBB		(3)	- Quantidade (Campo: QTPORCAOVND)
            D		(1)	- Unidade da Porção (Campo: IDUNPORCAOVND)
            EE		(2)	- Parte Inteira da Medida Caseira (Campo: QTMEDCASVND)
            F		(1)	- Parte Decimal da Medida Caseira (Campo: IDPADEMEDCASVND)
            GG		(2)	- Medida Caseira Utilizada (Campo: IDTIPOMEDCASVND)
            EEEE	(4)	- Valor Energético (Campo: VRENRGNUTRI)
            IIII	(4)	- Carboidatros (Campo: QTDCARBNUTRI)
            LLL		(3)	- Proteínas (Campo: QTDPROTNUTRI)
            NNN		(3)	- Gorduras Totais (Campo: QTDGTOTNUTRI)
            OOO		(3)	- Gorduras Saturadas (Campo: QTDGSATNUTRI)
            PPP		(3)	- Gorduras Trans (Campo: QTDGTRANUTRI)
            QQQ		(3)	- Fibra Alimentar (Campo: QTDFIBRANUTRI)
            UUUUU	(5)	- Sódio (Campo: QTDSODNUTRI)
        */
    }
    
    public function checkField($row, $field, $size){
        // conferindo se o parametro existe na linha
        if(array_key_exists($field, $row)){
            $retorno = $row[$field];
            if($retorno){
                //garantindo que o parametro tenha no máximo o tamanho especificado
                $retorno = substr($retorno, 0, $size);
                // caso o parametro não tenha o tamanho especificado
                if(strlen($retorno) < $size){
                    // adiciono zeros a esquerda até atingir o valor especificado
                    $retorno = str_pad($retorno, $size, '0', STR_PAD_LEFT);
                }
            }else{
                $retorno = str_repeat('0', $size);    
            }
        }else{
            $retorno = str_repeat('0', $size);
        }
        return $retorno;
    }
}