<?php

namespace Teknisa\MAN\Factories\Retail;

use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class ImpostosItemPreco extends BaseRepo {
    // Delphi function equivalent = TotalImpostos
    public function getTotalImpostos($CDFILIAL, $CDARVPROD, $NRORG) {
        $doVrAliq = 0;
        
        // Gera registros na tabela INCITEMP
        $this->procedureConsultaIncidencia($CDFILIAL, $CDARVPROD, $NRORG);
        
        // Gera registros na tabela ALIQTEMP
        $this->procedureConsultaAliquota($CDFILIAL, $CDARVPROD, $NRORG);
        
        $doVrAliq += $this->getVRALIQUTMP($CDFILIAL);
        
        // Limpar as tabelas temporárias que as procedures usaram para gerar valor.
        $this->deleteVRALIQUTMP($CDFILIAL);
        $this->deleteINCITEMP  ($CDFILIAL);
        
        return $doVrAliq;
    }
    
    private function procedureConsultaIncidencia($CDFILIAL, $CDARVPROD, $NRORG) {
        $procedure = new StoredProcedure($this->conn, 'CONSULTA_INCIDENCIA');
        
        $adqFilial = $this->getAdqFilial($CDFILIAL, $NRORG);
        
        $CDEMPRESA  = $adqFilial['CDEMPRESA'];
        $CDINSCESTA = $adqFilial['CDINSCESTA'];
        $CHAVE      = $this->getChaveFromCDFILIAL($CDFILIAL);

        $procedure->addParam(new Param('P_ENTRSAIDIN',   Param::PARAM_INPUT,  'S',         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_EMPRESA',      Param::PARAM_INPUT,  $CDEMPRESA,  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_PRODUTO',      Param::PARAM_INPUT,  $CDARVPROD,  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_INSCESTA',     Param::PARAM_INPUT,  $CDINSCESTA, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_TIPOOPER',     Param::PARAM_INPUT,  '01',        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_FILIAL',       Param::PARAM_INPUT,  $CDFILIAL,   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_LANCTONF',     Param::PARAM_INPUT,  $CHAVE,      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDGRUPOFISC',  Param::PARAM_INPUT,  NULL,        Param::PARAM_TYPE_NULL));
        
        return $procedure->execute();
    }
    
    private function procedureConsultaAliquota($CDFILIAL, $CDARVPROD, $NRORG) {
        $procedure = new StoredProcedure($this->conn, 'CONSULTA_ALIQUOTA');
        
        $adqFilial = $this->getAdqFilial($CDFILIAL, $NRORG);
        
        $CDEMPRESA      = $adqFilial['CDEMPRESA'];
        $CDINSCESTA     = $adqFilial['CDINSCESTA'];
        $CDPAIS         = $adqFilial['CDPAIS'];
        $SGESTADO       = $adqFilial['SGESTADO'];
        $CDMUNICIPIO    = $adqFilial['CDMUNICIPIO'];
        
        $CDIMPOTEMP     = $this->getCDIMPOTEMP($CDFILIAL);
        $CHAVE          = $this->getChaveFromCDFILIAL($CDFILIAL); 
        $DTINIVALID     = date("d/m/Y");
        
        $procedure->addParam(new Param('P_IMPOSTO',      Param::PARAM_INPUT,  $CDIMPOTEMP,   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_ENTRSAIDAL',   Param::PARAM_INPUT,  'S',           Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_EMPRESA',      Param::PARAM_INPUT,  $CDEMPRESA,    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_PRODUTO',      Param::PARAM_INPUT,  $CDARVPROD,    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_INSCESTA',     Param::PARAM_INPUT,  $CDINSCESTA,   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_PAISORIG',     Param::PARAM_INPUT,  $CDPAIS,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_ESTAORIG',     Param::PARAM_INPUT,  $SGESTADO,     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_PAISDEST',     Param::PARAM_INPUT,  $CDPAIS,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_ESTADEST',     Param::PARAM_INPUT,  $SGESTADO,     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_MUNIORIG',     Param::PARAM_INPUT,  $CDMUNICIPIO,  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_BASECALC',     Param::PARAM_INPUT,  0,             Param::PARAM_TYPE_INT));
        $procedure->addParam(new Param('P_INIVALID',     Param::PARAM_INPUT,  $DTINIVALID,   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_FILIAL',       Param::PARAM_INPUT,  $CDFILIAL,     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_LANCTONF',     Param::PARAM_INPUT,  $CHAVE,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDGRUPOFISC',  Param::PARAM_INPUT,  NULL,          Param::PARAM_TYPE_NULL));
        $procedure->addParam(new Param('P_TIPOOPER',     Param::PARAM_INPUT,  '01',          Param::PARAM_TYPE_STR));
        
        return $procedure->execute();
    }
    
    private function getAdqFilial ($CDFILIAL, $NRORG) {
        $params = array(
            ':CDFILIAL'       =>  $CDFILIAL,
            ':NRORG'          =>  $NRORG
        );
        
        $result = $this->conn->fetchAll(self::GET_ADQFILIAL, $params);
        
        return empty($result) ? null : $result[0];
    }
    
    private function getChaveFromCDFILIAL($CDFILIAL) {
        return str_pad($CDFILIAL, 10, '0', STR_PAD_LEFT);
    }
    
    private function getCDIMPOTEMP($CDFILIAL) {
        $CHAVE = $this->getChaveFromCDFILIAL($CDFILIAL);
        
        $params = array(
            ':CHAVE' => $CHAVE  
        );
        
        $result = $this->conn->fetchAll(self::GET_CDIMPOTEMP, $params);
        
        return empty($result) ? null : $result[0]['CDIMPOTEMP'];
    }
    
    private function getVRALIQUTMP($CDFILIAL) {
        $CHAVE = $this->getChaveFromCDFILIAL($CDFILIAL);
        
        $params = array(
            ':CHAVE'  =>  $CHAVE,
        );
        
        $result = $this->conn->fetchAll(self::GET_VRALIQTMP, $params);
        
        return empty($result) ? null : $result[0]['VRALIQTMP'];
    } 
    
    private function deleteVRALIQUTMP($CDFILIAL) {
        $CHAVE = $this->getChaveFromCDFILIAL($CDFILIAL);
        
        $params = array(
            ':CHAVE'  =>  $CHAVE,
        );
        
        $this->conn->executeQuery(self::DELETE_ALIQTEMP, $params);
    } 
    
    private function deleteINCITEMP($CDFILIAL) {
        $CHAVE = $this->getChaveFromCDFILIAL($CDFILIAL);
        
        $params = array(
            ':CHAVE'  =>  $CHAVE,
        );
        
        $this->conn->executeQuery(self::DELETE_INCITEMP, $params);
    } 
    
    
    const GET_ADQFILIAL = "
    	SELECT DISTINCT
    		   FI.CDFILIAL  , FI.NMFILIAL, FI.CDSEGMER, FI.CDEMPRESA,
    		   FI.CDINSCESTA, E.CDPAIS   , E.SGESTADO , E.CDMUNICIPIO,
    		   SE.CDSEGMER||' - '||SE.DSSEGMER AS SEGMENTO,
    		   PA.IDPRECDIFCOM
    	  FROM
    		   FILIAL FI
          LEFT JOIN SEGMERCADO SE ON
            (FI.CDSEGMER = SE.CDSEGMER)AND
            (FI.NRORG    = SE.NRORG)
          LEFT JOIN PARAVEND PA ON
            (FI.CDFILIAL = PA.CDFILIAL) AND
            (FI.NRORG    = PA.NRORG)
          LEFT JOIN ENDEFILI E ON
            (FI.CDFILIAL = E.CDFILIAL) AND
            (FI.NRORG    = E.NRORG)
    	  WHERE 
    		  (FI.CDFILIAL = :CDFILIAL) AND
             (FI.NRORG    = :NRORG)	  
	";
	
	const GET_CDIMPOTEMP = "
    	SELECT CDIMPOTEMP
          FROM INCITEMP
         WHERE NRINCITEMP = :CHAVE 
           AND CDFISCTEMP = '1'
    ";
    
    const GET_VRALIQTMP = "
        SELECT VRALIQUTMP
          FROM ALIQTEMP
         WHERE NRALIQTEMP = :CHAVE
    ";

    const DELETE_ALIQTEMP = "
        DELETE ALIQTEMP
         WHERE NRALIQTEMP = :CHAVE
    ";
    
    const DELETE_INCITEMP = "
        DELETE INCITEMP
         WHERE NRINCITEMP = :CHAVE 
           AND CDFISCTEMP = '1'
    ";
}